package ejava.ejb.mdb;

import java.rmi.RemoteException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.jms.Queue;
import javax.jms.QueueSender;
import javax.jms.Session;
import java.util.Date;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import ejava.servlets.Address;
import ejava.servlets.Customer;
import ejava.servlets.Item;
import ejava.servlets.TShirt;
import ejava.servlets.Order;

public class BatchOrderClient {
  // Client JMS producer interfaces
  private QueueConnection connection;
  private QueueSession requesterSession;
  private QueueSender requester;
  private Queue requesterQueue;

  /**
   * Client constructor does the following standard
   * operations for a JMS producer:
   *    - Get JNDI context
   *    - Lookup JMS connection factory from JNDI
   *    - Create JMS connection
   *    - Create JMS session
   *    - Lookup JMS queue name from JNDI
   *    - Create a JMS sender on the queue
   *    - Set local JMS producer variables
   *    - Start the JMS connection
   */
  public BatchOrderClient(String connectionFactoryName,
                          String queueName)	throws Exception
  {
    InitialContext context = new InitialContext();
    QueueConnectionFactory queueConnectionFactory =
      (QueueConnectionFactory) context.lookup(connectionFactoryName);
    QueueConnection connection =
      queueConnectionFactory.createQueueConnection();
    QueueSession requesterSession =
      connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
    Queue requesterQueue = (Queue) context.lookup(queueName);
    QueueSender requester =
      requesterSession.createSender(requesterQueue);
    set(connection, requesterSession, requester, requesterQueue);
    connection.start();
  }

  /** Set local JMS producer variables */
  private void set(QueueConnection connection,
                   QueueSession senderSession,
                   QueueSender sender, Queue senderQueue)
  {
    this.connection = connection;
    this.requesterSession = senderSession;
    this.requester = sender;
    this.requesterQueue = senderQueue;
  }

  /**
   * Create an ObjectMessage, create an Order,
   * set the Order onto the ObjectMessage, then
   * send the message over the JMS producer interface.
   */
  protected void writeMessage() throws JMSException {
    // Create a Message
    ObjectMessage message = requesterSession.createObjectMessage();
    // Create an Order
    Order order = createNewOrder();
    // Set the message body
    message.setObject(order);
    System.out.println("Sending Order :");
    // Produce message
    this.requester.send(message);
  }

  /* Create an example order */
  private Order createNewOrder() {
    Order order = new Order();
    // Order is created today and paid today and shipped today.
    order.setOrderDate(new Date());
    order.setPaidDate(new Date());
    order.setShippingInstructions("Leave at Door");
    order.setShipWeight(10.8);
    order.setShipCharge(32.54);
    order.setShipDate(new Date());
    // The customer info for the order is created
    Customer customer = new Customer();
    int aCustomerID = 101;
    customer.setCustomerID("" + aCustomerID);
    order.setCustomer(customer);
    order.setBillingAddress(createAnAddress());
    order.setShippingAddress(createAnAddress());
    order.setItems(createItems());
    return order;
  }

  /* Create an example address */
  private Address createAnAddress() {
    Address address = new Address();
    address.setAddress_1(" 2.1 EJB Way");
    address.setAddress_2(" 1");
    address.setCity(" J2EE ");
    address.setState("CA");
    address.setZipCode("14000");
    return address;
  }

  /* Create an example set of order items */
  private TShirt[] createItems() {
    TShirt itemOne = new TShirt();
    itemOne.setQuantity(2);
    itemOne.setTotalPrice(21.0);
    itemOne.setSize("XL");
    itemOne.setColor("White");
    itemOne.setUnitPrice(10.5);

    TShirt itemTwo = new TShirt();
    itemTwo.setQuantity(2);
    itemTwo.setTotalPrice(21.0);
    itemTwo.setSize("L");
    itemTwo.setColor("White");
    itemTwo.setUnitPrice(10.5);
    TShirt[] createdItems = { itemOne, itemTwo };
    return createdItems;
  }

  /**
   * Create a new client object with a JMS connection factory
   * name, and queue name passed in from the command line.
   *
   * Then write a message to the consumer after user hits
   * the "enter" key on the command line.
   */
  public static void main(String[] args) throws Exception {
    if (args.length < 1)
      System.out.println("Queue missing");
    BatchOrderClient sender = new BatchOrderClient(args[0], args[1]);
    BufferedReader commandLineReader =
      new java.io.BufferedReader(new InputStreamReader(System.in));
    sender.writeMessage();
  }
}
