package ejava.jaxp;



import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Entity;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Notation;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


/**
  * this example reads the document and prints the contents
  * to standard out
  */
public class DOMExample
{
  static final String SCHEMA_PROP =
              "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
  static final String SCHEMA_PROP_VALUE =
              "http://www.w3.org/2001/XMLSchema";
  static final String SCHEMA_SOURCE_PROP =
              "http://java.sun.com/xml/jaxp/properties/schemaSource";

  /**
  * This method prints the Node Name and Node Value
  */
  public static void printNodeNameAndValue(Node node)
          throws DOMException
  {
    System.out.println("Name :" + node.getNodeName());
    System.out.println("Value :" + node.getNodeValue());

    if(node.hasChildNodes()) {
      NodeList nodeList = node.getChildNodes();

      // print all child node information.
      for(int i = 0; i < nodeList.getLength(); i++) {
        Node item = nodeList.item(i);

        System.out.println(" Node Name :" + item.getNodeName());
        System.out.println(" Node Value :" + item.getNodeValue());
        printNodeNameAndValue(item);
      }
    }
  }

  /**
    * print Attribute information
    */
  public static void printAttributeInformation(Attr attribute)
  {
    System.out.println("Attibute Name :" + attribute.getName());
    System.out.println("Is it specified :" + attribute.getSpecified());
    System.out.println("Attribute Value :" + attribute.getValue());
    System.out.println("Owning Element Local Name :" 
                + attribute.getOwnerElement().getLocalName());    
  }

  /**
    * Entity information
    */
  public static void printEntityInformation(Entity entity)
  {
    System.out.println("Entity Name :" + entity.getNotationName());
    System.out.println("Entity System ID :" + entity.getSystemId());
    System.out.println("Entity Public ID :" + entity.getPublicId());
  }

  /**
    * Processing Instruction information
    */
  public static void printProcessingInstructionInformation(
          ProcessingInstruction processingInstruction)
  {
    System.out.println("Data  :" + processingInstruction.getData());
    System.out.println("Target  :" + processingInstruction.getTarget());
  }

  /**
  *  Get character data
  */
  public static void printCharacterData(CharacterData charData)
  {
    System.out.println("Character Data  :" + charData.getData());
  }

  /**
    * Notation information
    */
  public static void printNotationInformation(Notation notation)
  {
    System.out.println("Notation System ID :" + notation.getSystemId());
    System.out.println("Notation Public ID :" + notation.getPublicId());
  }

  /**
  * iterates through a NameNodeMap and prints all the elements
  */
  public static void printNameNodeMap(NamedNodeMap values)
          throws SAXParseException
  {
    int length = values.getLength();

    for(int i = 0; i < length; i++) {
      Node  node     = values.item(i);
      short nodeType = node.getNodeType();

      switch(nodeType) {

      case Node.ATTRIBUTE_NODE : {
        System.out.println(" This is an attribute :");

        Attr attribute = (Attr) node;

        printAttributeInformation(attribute);
        printNodeNameAndValue(node);

        break;
      }
      case Node.CDATA_SECTION_NODE : {
        System.out.println("This is a CDATA section :");

        CDATASection cdataSection = (CDATASection) node;

        printCharacterData(cdataSection);
        printNodeNameAndValue(cdataSection);

        break;
      }
      case Node.COMMENT_NODE : {
        System.out.println("This is a comment :");

        Comment comment = (Comment) node;

        printCharacterData(comment);
        printNodeNameAndValue(comment);

        break;
      }
      case Node.DOCUMENT_FRAGMENT_NODE : {
        DocumentFragment documentFragment = (DocumentFragment) node;

        System.out.println("This is a document fragment :");
        printNodeNameAndValue(node);

        break;
      }
      case Node.DOCUMENT_NODE : {
        System.out.println("This is a document :");

        Document document = (Document) node;

        printNodeNameAndValue(node);

        break;
      }
      case Node.DOCUMENT_TYPE_NODE : {
        System.out.println("This is a document type :");

        DocumentType documentType = (DocumentType) node;

        printNodeNameAndValue(node);

        break;
      }
      case Node.ELEMENT_NODE : {
        System.out.println("This is an element :");

        Element element = (Element) node;

        printNodeNameAndValue(node);

        break;
      }
      case Node.ENTITY_NODE : {
        System.out.println("This is an entity :");

        Entity entity = (Entity) node;

        printEntityInformation(entity);
        printNodeNameAndValue(node);

        break;
      }
      case Node.ENTITY_REFERENCE_NODE : {
        System.out.println("This is an entity reference :");

        EntityReference entityReference = (EntityReference) node;

        printNodeNameAndValue(node);

        break;
      }
      case Node.NOTATION_NODE : {
        System.out.println(" This is a notation :");

        Notation notation = (Notation) node;

        printNotationInformation(notation);
        printNodeNameAndValue(node);

        break;
      }
      case Node.PROCESSING_INSTRUCTION_NODE : {
        System.out.println("This is a processing instruction :");

        ProcessingInstruction processingInstruction =
          (ProcessingInstruction) node;

        printProcessingInstructionInformation(processingInstruction);
        printNodeNameAndValue(node);

        break;
      }
      case Node.TEXT_NODE : {
        System.out.println("This is text :");

        Text text = (Text) node;

        printNodeNameAndValue(node);

        break;
      }
      default : {
        System.out.println("This is not a defined node.");

        break;
      }
      }
    }
  }

  /**
    * This method prints all the information about Document Type
    */
  public static void printDocumentTypeInformation(
          DocumentType documentType) throws SAXParseException
  {
    String documentName = documentType.getName();

    System.out.println("Document Name :" + documentName);

    //get all the entities that are defined for document.
    NamedNodeMap entities = documentType.getEntities();

    printNameNodeMap(entities);
  }

  /**
  * This methods prints the Document contents
  */
  public static void printDocument(Document document)
          throws SAXParseException
  {
    DocumentType documentType = document.getDoctype();

    // can be null if using an XML Schema document
    if (documentType != null)
      printDocumentTypeInformation(documentType);
    else
      System.out.println("No document type declared");
  
    Element element = document.getDocumentElement();

    printNodeNameAndValue(element);

    NamedNodeMap attributes = element.getAttributes();

    printNameNodeMap(attributes);
  }

  public static void main(String[] argv)
  {
    if(argv.length < 1) {
      System.out.print("Usage: Specify XML and optional schema file.");        
      System.out.println("java ejava.jaxp.DOMExample <xml> [schema]");      
      System.exit(0);
    }

    try {
      String xmlFileURI = "file:" + new File(argv[0]).getAbsolutePath();

      // get new instance of Document Factory
      DocumentBuilderFactory domFactory 
            = DocumentBuilderFactory.newInstance();
      // set namespace awareness true
      domFactory.setNamespaceAware(true); 
      // set document validation true
      domFactory.setValidating(true);

      // if using a schema for validation, then set schema 
      // property and source
      if(argv.length == 2){
          domFactory.setAttribute(SCHEMA_PROP, SCHEMA_PROP_VALUE);
          domFactory.setAttribute(SCHEMA_SOURCE_PROP, new File(argv[1]));
      }

      // get new Document Builder
      DocumentBuilder documentBuilder = domFactory.newDocumentBuilder();
      // parse the document
      Document document = documentBuilder.parse(xmlFileURI);
      // normalize the document
      document.getDocumentElement().normalize();
      // print out the document
      System.out.println("Root element is "
                         + document.getDocumentElement().getNodeName());
      printDocument(document);
    } catch(DOMException domParseException) {
      System.out.println(" Parsing error ");
      System.out.println(domParseException.getMessage());
      domParseException.printStackTrace();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
}
