package ejava.jaxr;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.jaxr
 * 
 * Type: JAXRClient.java
 * 
 */

import javax.xml.registry.*;
import javax.xml.registry.infomodel.*;

import java.net.PasswordAuthentication;
import java.util.*;
import java.util.prefs.*;

public class JAXRClient {
  
  // Constants
  public static String QUERY_URL_PROPNAME 
    = "javax.xml.registry.queryManagerURL";
  public static String LIFECYCLE_URL_PROPNAME 
    = "javax.xml.registry.lifeCycleManagerURL";

  // JAXR connection properties
  private String queryManagerURL = "";
  private String lifeCycleManagerURL = "";

  // Web Services Registry username and password
  private String registryUsername = "";
  private char[] registryPassword = null;

  // Registry Object Info
  private String orgName = "";
  private String orgDescription = "";
  private String orgPhone = "";

  private String user1Name = "";
  private String user1Type = "";
  private String phone1 = "";
  private String email1 = "";

  private String user2Name = "";
  private String user2Type = "";
  private String phone2 = "";
  private String email2 = "";

  private String serviceName = "";
  private String serviceDescription = "";
  private String serviceBindingDescription = "";
  private String serviceBindingURI = "";
  
  // Registry search name pattern
  private String namePattern = "";
  
  // Declarative query string
  private String queryString = "";
  
	/**
	 * Constructor for JAXRClient.
	 */
	public JAXRClient() {
    
    // Read preferences for the client
    Preferences prefs 
      = Preferences.userNodeForPackage(ejava.jaxr.JAXRClient.class);

    queryManagerURL = prefs.get("QUERY_MANAGER_URL", "");
    lifeCycleManagerURL = prefs.get("LIFECYCLE_MANAGER_URL", "");
    
    registryUsername = prefs.get("REGISTRY_USERNAME", "");
    registryPassword 
      = prefs.get("REGISTRY_PASSWORD", "").toCharArray();

    orgName = prefs.get("ORG_NAME", "");
    orgDescription = prefs.get("ORG_DESCRIPTION", "");
    orgPhone = prefs.get("ORG_PHONE", "");
   
    user1Name = prefs.get("USER1_NAME", "");
    user1Type = prefs.get("USER1_TYPE", "");
    phone1 = prefs.get("PHONE1", "");
    email1 = prefs.get("EMAIL1", "");
    
    user2Name = prefs.get("USER2_NAME", "");
    user2Type = prefs.get("USER2_TYPE", "");
    phone2 = prefs.get("PHONE2", "");
    email2 = prefs.get("EMAIL2", "");
    
    serviceName = prefs.get("SERVICE_NAME", "");
    serviceDescription = prefs.get("SERVICE_DESCRIPTION", "");
    serviceBindingDescription = prefs.get("SERVICE_BINDING_DESCRIPTION", "");
    serviceBindingURI = prefs.get("SERVICE_BINDING_URI", "");
  
    namePattern = prefs.get("NAME_PATTERN", "");

    queryString = prefs.get("QUERY_STRING", "");
  }

  /**
   * Helper for handling exceptions
   */
  public void handleException(Exception e){
    System.out.println("\nJAXRClient Exception.\n" 
                        + e.getMessage() + "\n");
    e.printStackTrace();
  }

  /**
   * Create a JAXR Connection
   */
  public Connection getConnection(){
    Connection conn = null;

    System.out.println("Setting connection factory with:\n"
                        + " - " + QUERY_URL_PROPNAME
                        + " = " + queryManagerURL
                        + " \n - " + LIFECYCLE_URL_PROPNAME
                        + " = " + lifeCycleManagerURL + "\n");
    
    try{
      // Get a connection factory
      ConnectionFactory cf = ConnectionFactory.newInstance();
            
      // Set connection factory properties
      Properties cfProps = new Properties();
      cfProps.setProperty(QUERY_URL_PROPNAME, queryManagerURL);
      cfProps.setProperty(LIFECYCLE_URL_PROPNAME, lifeCycleManagerURL);
      cf.setProperties(cfProps);
      
      // Create the connection
      conn = cf.createConnection();
    }catch(JAXRException e){
      handleException(e);
    }
    
    return conn;
  }
  
  /**
   * Set the user credentials onto the connection
   */
  public void authenticate(Connection conn){
    try{
      // Create a java.net.PasswordAuthentication object
      //    with the registry username and password
      PasswordAuthentication credential
        = new PasswordAuthentication(registryUsername, 
                                      registryPassword);
      // Create a credentials Set
      Set credentials = new HashSet();
      credentials.add(credential);
      // Set credentials onto JAXR connection
      conn.setCredentials(credentials);                   
    }catch(JAXRException e){
      handleException(e);
    }
  }
  
  /**
   * Get a BusinessQueryManager handle
   */
  public BusinessQueryManager getBusinessQueryManager(Connection conn){
    BusinessQueryManager queryMgr = null;    
    try{
      RegistryService rs = conn.getRegistryService();
      queryMgr = rs.getBusinessQueryManager();
    }catch(JAXRException e){
      handleException(e);
    }  
    return queryMgr;
  }

  /** 
   * Get a DeclarativeQueryManager handle
   */
  public DeclarativeQueryManager 
  getDeclarativeQueryManager(Connection conn){
    DeclarativeQueryManager queryMgr = null;    
    try{
      RegistryService rs = conn.getRegistryService();
      queryMgr = rs.getDeclarativeQueryManager();
    }catch(JAXRException e){
      handleException(e);
    }  
    return queryMgr;
  }

  /**
   * Get a BusinessLifeCycleManager handle
   */
  public BusinessLifeCycleManager 
  getBusinessLifeCycleManager(Connection conn){
    BusinessLifeCycleManager lifecycleMgr = null;    
    try{
      RegistryService rs = conn.getRegistryService();
      lifecycleMgr = rs.getBusinessLifeCycleManager();
    }catch(JAXRException e){
      handleException(e);
    }  
    return lifecycleMgr;
  }

  /**
   * Traverse a BulkResponse object and print out its contents.
   */
  public void processBulkResponse(BulkResponse response){
    // Check for null BulkResponse
    if(response == null){
      System.out.println("\n  -- NULL BulkResponse");
      return;
    }else{
      System.out.println("\n  -- Processing BulkResponse...");
    }
    
    try{
      // Get info from the bulk response
      String id = response.getRequestId();
      int status = response.getStatus();
      boolean available = response.isAvailable();
      Collection objCollection = response.getCollection();
      Collection exCollection = response.getExceptions();
      boolean partial = response.isPartialResponse();
      
      // Display the info
      System.out.println("    Response with ID " + id
                        + " has status " + status
                        + " and isAvailable as " + available
                        + " and isPartialResponse as " + partial);
      
      // If the response collection is not null, print it out     
      if(objCollection != null){
        Iterator objects = objCollection.iterator();
        System.out.println("  - Contents of response...");
        while(objects.hasNext()){
          Object obj = objects.next();
          System.out.println("    -- Response object of type " 
                              + obj.getClass().getName());
                              
          processBusinessObject(obj);          
        }
      }

      // If the exception collection is not null, print it out
      if(exCollection != null){
        Iterator exceptions = exCollection.iterator();
        System.out.println("- Exceptions in response...");
        while(exceptions.hasNext()){
          Object obj = exceptions.next();
          if(obj instanceof Exception){
            Exception ex = (Exception) obj;
                     
            System.out.println("-- Exception is " + ex.getMessage());
          }
        }
      }
    }catch(JAXRException e){
      handleException(e);
    }    
  }
  
  /**
   * Process a business object by printing its info
   */
  public void processBusinessObject(Object obj){
    try{
      if(obj instanceof Organization){ // Print Organization info
        Organization org = (Organization) obj;
  
        String key = org.getKey().getId();
        String name = org.getName().getValue();
        String desc = org.getDescription().getValue();
        
        System.out.println("       Organization with ID " + key
                            + " and with name " + name
                            + " and description " + desc);
                            
        Iterator users = org.getUsers().iterator();
        while(users.hasNext()){
          User user = (User) users.next();
          System.out.println("         User name = " 
                             + user.getPersonName().getFullName());
        }
      }
      else if(obj instanceof Key){ // Print out a Key
        Key key = (Key) obj;           
        System.out.println("       Key with ID " + key.getId());
      }
    }catch(JAXRException e){
      handleException(e);
    } 
  }

  /**
   * Create a registry object
   */
  public BulkResponse createObject(BusinessLifeCycleManager lcm){
    BulkResponse resp = null;

    System.out.println("\n~~~~ Creating an object\n");

    try {
      // Create Organization object from configurable info
      Organization organization = createOrganization(lcm);
      
      // Create Collection of Organization objects
      Collection organizations = new ArrayList();
      organizations.add(organization);
  
      // Save Organization updates to the registry
      resp = lcm.saveOrganizations(organizations);
    } catch (JAXRException e) {
        handleException(e);
    }
    
    return resp;
  }

  /**
   * Create an Organization object using configurable data
   */
  public Organization createOrganization(BusinessLifeCycleManager lcm){
    Organization org = null;
    try{
      // Organization
      org = lcm.createOrganization(orgName);

      // Organization description
      InternationalString od 
        = lcm.createInternationalString(orgDescription);
      org.setDescription(od);
         
      // User1 Info (is a primary user too)
      User u1 = lcm.createUser();
      // Person name for user1            
      PersonName pn1 = lcm.createPersonName(user1Name);
      u1.setPersonName(pn1);
      u1.setType(user1Type);
      // Phone for user1
      TelephoneNumber tn1 = lcm.createTelephoneNumber();
      tn1.setNumber(phone1);
      Collection tns1 = new ArrayList();
      tns1.add(tn1);
      u1.setTelephoneNumbers(tns1);
      // Email for user1
      EmailAddress e1 = lcm.createEmailAddress(email1);
      Collection es1 = new ArrayList();
      es1.add(e1);
      u1.setEmailAddresses(es1);
  
      // User2 Info (is a primary user too)
      User u2 = lcm.createUser();
      // Person name for user2            
      PersonName pn2 = lcm.createPersonName(user2Name);
      u2.setPersonName(pn2);
      u2.setType(user2Type);
      // Phone for user2
      TelephoneNumber tn2 = lcm.createTelephoneNumber();
      tn2.setNumber(phone2);
      Collection tns2 = new ArrayList();
      tns2.add(tn2);
      u2.setTelephoneNumbers(tns2);
      // Email for user2
      EmailAddress e2 = lcm.createEmailAddress(email2);
      Collection es2 = new ArrayList();
      es2.add(e2);
      u2.setEmailAddresses(es2);  
  
      // Add users and primary contact to the organization
      org.setPrimaryContact(u1);
      org.addUser(u1);
      org.addUser(u2);
 
      // Service
      Service s = lcm.createService(serviceName);
      InternationalString sd 
        = lcm.createInternationalString(serviceDescription);
      s.setDescription(sd);
  
      // Service bindings
      ServiceBinding b = lcm.createServiceBinding();
      InternationalString sbd 
        = lcm.createInternationalString(serviceBindingDescription);
      b.setDescription(sbd);
      b.setValidateURI(false); // don't validate link since phony
      b.setAccessURI(serviceBindingURI);
      Collection sbs = new ArrayList();
      sbs.add(b);
    
      // Add service binding to the service
      s.addServiceBindings(sbs);
  
      // Add service to the organization
      Collection ss = new ArrayList();
      ss.add(s);
      org.addServices(ss);    
    }catch(JAXRException e){
      handleException(e);
    }
    return org;
  }

  /**
   * Lookup an object in the registry
   */
  public BulkResponse lookupObject(BusinessQueryManager bqm){
    BulkResponse resp = null;

    System.out.println("\n~~~~ Looking up an object\n");

    try{
      // Create find qualifiers
      Collection quals = new ArrayList();
      quals.add(FindQualifier.SORT_BY_NAME_ASC);
      // Create a name pattern
      Collection names = new ArrayList();    
      names.add(namePattern);
      // Now induce the finder method query
      resp 
        = bqm.findOrganizations(quals, names, null, null, null, null);
    }catch(JAXRException e){
      handleException(e);
    }
    return resp;
  }
  
  /**
   * Lookup a collection of keys in the registry
   */
  public Collection lookupKeys(BusinessQueryManager bqm){
    Collection keys = new ArrayList();

    System.out.println("\n~~~~ Looking up a key\n");

    try{      
      // First lookup the object
      BulkResponse resp = lookupObject(bqm);
      // Iterate through the response collection
      Collection objCollection = resp.getCollection();
      Iterator objects = objCollection.iterator();
      while(objects.hasNext()){
        // Add the organization key to the collection
        Organization org = (Organization) objects.next();
        keys.add(org.getKey());
      }
    }catch(JAXRException e){
      handleException(e);
    }
    return keys;
  }  

  /**
   * Lookup an object in the registry by its key
   */
  public BulkResponse lookupObjectsByKey(QueryManager qm,
                                          Collection keys){
    BulkResponse resp = null;

    System.out.println("\n~~~~ Looking up an object by key\n");

    try{
      // Lookup Organization objects with keys
      resp = qm.getRegistryObjects(keys,LifeCycleManager.ORGANIZATION);
    }catch(JAXRException e){
      handleException(e);
    }
    return resp;
  }

  /**
   * Delete an object from the registry
   */
  public BulkResponse deleteObject(BusinessLifeCycleManager lcm, Collection keys){
    BulkResponse resp = null;

    System.out.println("\n~~~~ Deleting an object\n");
    
    try{
      // Delete the organizations associated with the keys
      resp = lcm.deleteOrganizations(keys);
    }catch(JAXRException e){
      handleException(e);
    }    
    return resp;
  }

  /**
   * Lookup an object in the registry by its key
   */
  public BulkResponse lookupDeclaratively(DeclarativeQueryManager dqm){
    BulkResponse resp = null;

    System.out.println("\n~~~~ Looking up an object declaratively\n");

    try{
      Query query = dqm.createQuery(Query.QUERY_TYPE_SQL, queryString);
      resp = dqm.executeQuery(query);
    }catch(JAXRException e){
      handleException(e);
    }
    return resp;
  }

  /**
   * Lookup CapabilityProfile
   */
  public CapabilityProfile getCapabilityProfile(Connection conn){
    CapabilityProfile cp = null;
    try{
      RegistryService rs = conn.getRegistryService();
      cp = rs.getCapabilityProfile();
    }catch(JAXRException e){
      handleException(e);
    }
    return cp;
  }

  /**
   * Main executable method for JAXR client
   */
	public static void main(String[] args) {
    
    System.out.println("\n\n**** Start JAXRClient ****\n\n");
    
    // Create an example JAXR client handle
    JAXRClient jaxr = new JAXRClient();

    try{
      // Get a JAXR connection
      Connection conn = jaxr.getConnection();
      // Authenticate self with connection
      jaxr.authenticate(conn);
      
      // Lookup a BusinessLifeCycleManager
      BusinessLifeCycleManager lifecycleMgr 
        = jaxr.getBusinessLifeCycleManager(conn);
              
      // Lookup a BusinessQueryManager
      BusinessQueryManager businessQueryMgr 
        = jaxr.getBusinessQueryManager(conn);
      
      // Create an object in the registry
      BulkResponse createdObjects
        = jaxr.createObject(lifecycleMgr);
      jaxr.processBulkResponse(createdObjects);

      // Lookup the object in the registry
      BulkResponse lookupObjects 
        = jaxr.lookupObject(businessQueryMgr);
      jaxr.processBulkResponse(lookupObjects);
     
      // Lookup the key for the object in the registry
      //   and then lookup objects by key
      Collection lookupKeys
        = jaxr.lookupKeys(businessQueryMgr);
      BulkResponse lookupObjectsByKey
        = jaxr.lookupObjectsByKey(businessQueryMgr, lookupKeys);
      jaxr.processBulkResponse(lookupObjectsByKey);
      
      // Lookup Object declaratively (if supported)
      CapabilityProfile cp = jaxr.getCapabilityProfile(conn);
      if(cp.getCapabilityLevel() == 1){
        DeclarativeQueryManager dqm
          = jaxr.getDeclarativeQueryManager(conn);
        BulkResponse declarativeLookup
          = jaxr.lookupDeclaratively(dqm);
        jaxr.processBulkResponse(declarativeLookup);
      }
        
      // Delete the object in the registry
      BulkResponse deleteObjectKeys
        = jaxr.deleteObject(lifecycleMgr, lookupKeys);
      jaxr.processBulkResponse(deleteObjectKeys);    

      // Try to lookup the object in the registry
      BulkResponse lookupNoObjects 
        = jaxr.lookupObject(businessQueryMgr);
      jaxr.processBulkResponse(lookupNoObjects);
      
      // Close the connection
      conn.close();
    }catch(Exception e){
      jaxr.handleException(e);
    }

    System.out.println("\n\n**** End JAXRClient ****\n\n");
	}
}
