package ejava.jdbc.advanced;

import java.util.prefs.Preferences;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.lang.ClassNotFoundException;
import java.sql.PreparedStatement;
import java.sql.BatchUpdateException;
import java.sql.DatabaseMetaData;

/**
 * Class BatchUpdatesExample, an example to show BatchUpdates
 */
public class BatchUpdatesExample extends ConnectToDatabase
{
  
  /**
   * Constructor
   */
  public BatchUpdatesExample()
  {
    super();
  }
  
  
  /**
   * if it is not connected to database, connect to database and
   * create a statement and returns the Statement
   * @param scrollType     should be ResultSet.TYPE_FORWARD_ONLY
   * or  ResultSet.TYPE_SCROLL_INSENSITIVE
   * or  ResultSet.TYPE_SCROLL_SENSITIVE
   * @param updateType     should be ResultSet.CONCUR_READ_ONLY  or
   * ResultSet.CONCUR_UPDATABLE ;
   */
  protected Statement getStatement(int scrollType, int updateType)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Statement statement =
      connection.createStatement(scrollType,updateType);
    
    return statement;
  }
  
  /**
   * if it is not connected to database, connect to database and
   * create a statement and returns the Statement.
   */
  protected Statement getStatement()
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    Statement statement = connection.createStatement();
    
    return statement;
  }
  
  /**
   * if it is not connected to database, connect to database and
   * create a PreparedStatement and returns the PreparedStatement.
   */
  protected PreparedStatement getStatement(String sqlString)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlString);
    
    return preparedStatement;
  }
  
  /**
   * if it is not connected to database, connect to database and
   * create a statement and returns the PreparedStatement.
   */
  protected PreparedStatement getStatement(String sqlString,
    int scrollType, int updateType)
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlString, scrollType, updateType);
    
    return preparedStatement;
  }
  
  /**
   * insert few rows at a time using statement object.
   */
  public void insertBatchofRowsUsingStatement()
    throws SQLException, BatchUpdateException, Exception
  {
    Connection connection = super.getConnection();
    DatabaseMetaData dbMetaData = connection.getMetaData();
    // check if driver supports batch updates
    if (dbMetaData.supportsBatchUpdates()) {
      try {
        connection.setAutoCommit(false);
        Statement statement = this.getStatement();
        // insert some rows
        statement.addBatch("INSERT INTO STATE VALUES(\'VA\', "
                             + "\'VIRGINIA')");
        statement.addBatch("INSERT INTO STATE VALUES(\'MD\',"
                             + " \'MARYLAND')");
        int[] nResults = statement.executeBatch();
        // commit
        connection.commit();
        
        // print results
        for (int i = 0; i < nResults.length; i++) {
          if(nResults[i] >= 0){
              System.out.println(i + "th statement executed with "+
                                   nResults[i] + " results");
          }
          else{
              System.out.println(i + "th statement executed with an"
                    + " indeterminable number of results."
                    + " return value = " + nResults[i]);            
          }
        }
      } catch (SQLException e) {
        try {
          // if failed to insert rollback
          connection.rollback();
        } catch (SQLException sqle) {
          throw new SQLException(sqle.getMessage(),
            sqle.getSQLState(), sqle.getErrorCode());
        }
        
        throw new SQLException(e.getMessage(),
            e.getSQLState(),  e.getErrorCode());
      }
    }
  }
  
  /**
   * insert few rows at a time using preparedStatement object.
   */
  public void insertBatchofRowsUsingPreparedStatement()
    throws SQLException, Exception
  {
    Connection connection = super.getConnection();
    DatabaseMetaData dbMetaData = connection.getMetaData();
    // check whether driver supports batchupdates
    if (dbMetaData.supportsBatchUpdates()) {
      try {
        connection.setAutoCommit(false);
        // get prepared statement
        PreparedStatement preparedStatement =
          this.getStatement("INSERT INTO STATE VALUES (?,?)");
        // set some statement data
        preparedStatement.setString(1, "CA");
        preparedStatement.setString(2, "California");
        preparedStatement.addBatch();
        preparedStatement.setString(1, "OH");
        preparedStatement.setString(2, "Ohio");
        // start batch
        preparedStatement.addBatch();
        // execute batch
        int[] nResults = preparedStatement.executeBatch();
        // commit batch
        connection.commit();

        // print results
        for (int i = 0; i < nResults.length; i++) {
          if(nResults[i] >= 0){
              System.out.println(i + "th statement executed with "+
                                   nResults[i] + " results");
          }
          else{
              System.out.println(i + "th statement executed with an"
                    + " indeterminable number of results."
                    + " return value = " + nResults[i]);            
          }
        }

      }
      catch (SQLException e) {
        try {
          // if failed to execute, rollback
          connection.rollback();
        } catch (SQLException sqle) {
          throw new SQLException(sqle.getMessage(),
          sqle.getSQLState(),sqle.getErrorCode());
        }
        throw new SQLException(e.getMessage(), e.getSQLState(),
          e.getErrorCode());
      }
    }
  }
  
  /**
   * Main method for testing
   */
  public static void main(String[] args)
  {
    // Read preferences and create example object
    Preferences preferences = Preferences.userNodeForPackage(
                             ejava.jdbc.advanced.ConnectToDatabase.class);
    
    BatchUpdatesExample example = new BatchUpdatesExample();
    
    try {
      // insert some rows as a batch using statement object
      example.insertBatchofRowsUsingStatement();
      // insert some rows as a batch using prepared statements
      example.insertBatchofRowsUsingPreparedStatement();
    }
    catch (ClassNotFoundException classNotFoundException) {
      String driverClass = preferences.get("DRIVER_CLASS",null);
      
      System.out.println("Driver Class :" + driverClass +
                           " Exception :" + classNotFoundException);
      System.exit(0);
    }
    catch (SQLException sqlExcetpion) {
      String dbURL = preferences.get("DATABASE_URL",null);
      System.out.println("Failed to connect db :" + dbURL +
         " Exception :" + sqlExcetpion);
      System.exit(0);
    } catch (Exception exception) {
      System.out.println("Failed to connect db :" + " Exception :"+
         exception);
      // if the exception is of batch type, print exception detail
      if (exception instanceof BatchUpdateException) {
        BatchUpdateException bupdateException =
          (BatchUpdateException) exception;
        System.out.println("SQL STATE :"+
                             bupdateException.getSQLState());
        System.out.println("Message  :" +
                             bupdateException.getMessage());
        System.out.println("Vendor  :"+
                             bupdateException.getErrorCode());
        System.out.println(" Update Counts :");
        int[] nRowsResult = bupdateException.getUpdateCounts();
        for (int i = 0; i < nRowsResult.length; i++) {
          System.err.println("n Resutls :" + nRowsResult[i]);
        }
      }
      System.exit(0);
    }
    
    System.exit(0);
  }
  
}


