package ejava.tcpip;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.DriverManager;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import java.util.prefs.Preferences;

/**
 * The DatabaseHelper is used to talk to the database
 * all the final sqlStatements are formed here.
 */
public class DatabaseHelper {
    private Connection          connection;
    private Statement           statement;
    private String              receiverHostName;
    private int                 receiverPort;
    private static final String CUSTOMER_QUERY =
        "SELECT C.* FROM TSHIRTS.CUSTOMER C, TSHIRTS.ORDERS O, "
        + "TSHIRTS.ITEM I WHERE C.CUSTOMER_ID = O.CUSTOMER_ID_FK AND "
        + "O.ORDER_ID = I.ORDER_ID_FK AND I.TOTAL_PRICE > ?";
    private static final String ORDER_QUERY =
        "SELECT ORDERS.order_id, ORDERS.order_date, ITEM.total_price"
        + " FROM TSHIRTS.ORDERS, TSHIRTS.ITEM, TSHIRTS.CUSTOMER WHERE "
        + "ORDERS.order_id = ITEM.order_id_fk AND "
        + "ORDERS.CUSTOMER_ID_FK = CUSTOMER.CUSTOMER_ID "
        + "AND CUSTOMER.state = ?";

    public DatabaseHelper() {}

    /**
     * To load the driver and make connection to the DB.
     */
    public void makeDbConnection() throws Exception {
        Preferences prefs =
            Preferences.userNodeForPackage(DatabaseHelper.class);

        // Get driver class, database URL, username, and password,
        // and then create DB Connection with this info
        String      driverClass = prefs.get("DRIVER_CLASS", null);

        Class.forName(driverClass);

        String dbUrl = prefs.get("DATABASE_URL", null);
        String userName = prefs.get("USER_NAME", null);
        String password = prefs.get("PASSWORD", null);

        if (userName == null) {
            connection = DriverManager.getConnection(dbUrl);
        } else {
            connection = DriverManager.getConnection(dbUrl, userName,
                                                    password);
        }
    }

    public void setConnection(Connection connection) {
        connection = connection;
    }

    public synchronized ResultSet getResultSet(String sqlQuery, Object whereValue)
            throws SQLException {

        // Create prepared statement, set where value, execute query
        PreparedStatement preparedStatement =
            connection.prepareStatement(sqlQuery);

        preparedStatement.setObject(1, whereValue);

        ResultSet rs = preparedStatement.executeQuery();

        return rs;
    }

    public synchronized Vector findCustomersWhoOrderdForMoreThan(float value)
            throws SQLException, Exception {

        // Get result set, meta data, and column count
        ResultSet         rs = getResultSet(CUSTOMER_QUERY,
                                            new Float(value));
        ResultSetMetaData rsMetaData = rs.getMetaData();
        int               nColumns = rsMetaData.getColumnCount();
        Vector            customers = null;

        // For each result, populate customer object with data
        while (rs.next()) {
            if (customers == null) {
                customers = new Vector();
            }

            String   customerID = rs.getString("CUSTOMER_ID");
            String   firstName = rs.getString("FIRST_NAME");
            String   lastName = rs.getString("LAST_NAME");
            String   address1 = rs.getString("ADDRESS_1");
            String   address2 = rs.getString("ADDRESS_2");
            String   city = rs.getString("CITY");
            String   state = rs.getString("STATE");
            String   phoneNumber = rs.getString("PHONE");
            String   eMail = rs.getString("EMAIL");
            String   zip = rs.getString("ZIP_CODE");
            Customer customer = new Customer();

            customer.setFirstName(firstName);
            customer.setLastName(lastName);
            customer.setAddress1(address1);
            customer.setAddress2(address2);
            customer.setCity(city);
            customer.setState(state);
            customer.setZip(zip);
            customer.setEmailAddress(eMail);
            customer.setPhoneNumber(phoneNumber);
            customers.addElement(customer);
        }

        return customers;
    }

    /**
     * Method
     *
     * get the orders which are order from a given state
     * @param state
     *
     * @throws SQLException
     *
     * @see
     */
    public synchronized Vector findOrdersWhichAreFrom(String state)
            throws SQLException, Exception {

        // Get result set from query, and column count
        ResultSet         rs = getResultSet(ORDER_QUERY, state);
        ResultSetMetaData rsMetaData = rs.getMetaData();
        int               nColumns = rsMetaData.getColumnCount();
        Vector            orders = null;

        // For each result, populate and Order object
        while (rs.next()) {
            if (orders == null) {
                orders = new Vector();
            }

            int    orderID = rs.getInt("ORDER_ID");
            Date   orderDate = rs.getDate("ORDER_DATE");
            double orderValue = rs.getDouble("TOTAL_PRICE");
            Order  order = new Order(orderID, orderDate.toString(),
                                     orderValue);

            orders.addElement(order);
        }

        return orders;
    }

    /**
     * Method
     * register ClientReceiver instance to the sever.
     * the ClientReceiver method will be class by the
     * OrderManagerImpl   of findOrdersWhichAreFrom();
     *
     * @param name
     * @param clientReceiver
     *
     * @throws RemoteException
     *
     * @see
     */
    public synchronized void register(String hostName, int port) {
        this.receiverHostName = hostName;
        receiverPort = port;
    }

}

