package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: SOAPClient
 * 
 */

import java.io.*;
import java.util.*;
import javax.xml.soap.*;
import java.net.URL;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import java.util.prefs.*;

public class SOAPClient extends SOAPManager {
	
  private String server = "localhost";
  private String port = "9000";

  /**
   * Create a SOAP client with server host and port
   */
  public SOAPClient(String serverHost, String serverPort) {
    server = serverHost;
    port = serverPort;
  }
  
  /**
   * Crates a message that is not understood by server
   */
  public SOAPMessage getInvalidMessage(){
    // Create empty message
    SOAPMessage message = createEmptyMessage();
    try{
        SOAPPart soapPart= message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        SOAPHeader header = createHeader(message, server, port);          
        SOAPBody body = getBody(message);
        
        Name name = envelope.createName("invalidMethod", 
                                    "ejava.webservices.NonExistentClass", 
                                    "http://beeshirts.com/soap");
        SOAPElement e1 = body.addChildElement(name);
        SOAPElement e2 = e1.addChildElement("invalidParameter");
        SOAPElement e3 = e2.addTextNode("blaBlaBla");
        
        message.saveChanges();    
    }
    catch(Exception e){
        printException(e);
    }
    return message;
  }

  /**
   * Creates a message containing query for Customer info
   */
  public SOAPMessage getCustomerMessage(){
    // Create empty message
    SOAPMessage message = createEmptyMessage();
    try{
        SOAPPart soapPart= message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        SOAPHeader header = createHeader(message, server, port);          
        SOAPBody body = getBody(message);
        
        Name name = envelope.createName("findCustomers", 
                                        "ejava.webservices.Customer", 
                                        "http://beeshirts.com/soap");
        SOAPElement e1 = body.addChildElement(name);
        SOAPElement e2 = e1.addChildElement("minimumOrder");
        SOAPElement e3 = e2.addTextNode("10");
        
        message.saveChanges();    
    }
    catch(Exception e){
        printException(e);
    }
    return message;
  }

  /**
   * Creates a message query for Order info
   */
  public SOAPMessage getOrderMessage(){
    // Create empty message
    SOAPMessage message = createEmptyMessage();
    try{
        SOAPPart soapPart= message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        SOAPHeader header = createHeader(message, server, port);          
        SOAPBody body = getBody(message);
        
        Name name = envelope.createName("findOrders", 
                                        "ejava.webservices.Order", 
                                        "http://beeshirts.com/soap");
        SOAPElement e1 = body.addChildElement(name);
        SOAPElement e2 = e1.addChildElement("state");
        SOAPElement e3 = e2.addTextNode("VA");
        
        message.saveChanges();    
    }
    catch(Exception e){
        printException(e);
    }
    return message;
  }
  
  /**
   * Creates a message that is a query for Coupon image
   */
  public SOAPMessage getCoupon(){
    // Create empty message
    SOAPMessage message = createEmptyMessage();
    try{
        SOAPPart soapPart= message.getSOAPPart();
        SOAPEnvelope envelope = soapPart.getEnvelope();
        SOAPHeader header = createHeader(message, server, port);          
        SOAPBody body = getBody(message);
        
        Name name = envelope.createName("getCoupon", 
                                        "ejava.webservices.Order", 
                                        "http://beeshirts.com/soap");
        SOAPElement e1 = body.addChildElement(name);
        SOAPElement e2 = e1.addChildElement("couponID");
        SOAPElement e3 = e2.addTextNode("BLDJEPRISE2");
        
        message.saveChanges();    
    }
    catch(Exception e){
        printException(e);
    }
    return message;
  }
  
  /**
   * Main method for connectnig to server, sending requests, and
   * getting responses. 
   */
  public static void main(String[] args) {
    	System.out.println("This is a SOAP client.");

        // Get the preferences       
        Preferences preferences = Preferences.userNodeForPackage(
                                      ejava.webservices.SOAPClient.class);
        String host = preferences.get("SERVER_HOST", "localhost");
        String port = preferences.get("SERVER_PORT", "9000");

        // Create a new SOAP client attached to server host and port
        SOAPClient client = new SOAPClient(host, port);

        try {
            // Create initial connection objects
            URL server = new URL("http://" + host + ":" + port);
            SOAPConnectionFactory connFactory 
                                = SOAPConnectionFactory.newInstance();
            SOAPConnection connection = connFactory.createConnection();            


            // Customer request and response
            SOAPMessage message = client.getCustomerMessage();
            System.out.println("\n\nSending Customer Query Message....");
            client.printMessage(message);
            SOAPMessage reply = connection.call(message, server);
            System.out.println("\n\nReceived Response Message....");
            client.printMessage(reply);

            // Order request and response
            message = client.getOrderMessage();
            System.out.println("\n\nSending Order Query Message....");
            client.printMessage(message);
            reply = connection.call(message, server);
            System.out.println("\n\nReceived Order Message....");
            client.printMessage(reply);


            // Send an invalid request and response
            message = client.getInvalidMessage();
            System.out.println("\n\nSending Invalid Query Message....");
            client.printMessage(message);
            reply = connection.call(message, server);
            System.out.println("\n\nReceived Invalid Message Response....");
            client.printMessage(reply);

            // Send a request generating a response w/attachment
            message = client.getCoupon();
            System.out.println("\n\nSending Coupon Query Message....");
            client.printMessage(message);
            reply = connection.call(message, server);
            System.out.println("\n\nReceived Coupon Message....");
            System.out.println("\n\nHit Ctrl-C to end application!");
            client.readAttachments(reply);
            System.in.read();
            
            // Close the connection
            connection.close();
        } catch(Exception e) {
            client.printException("SOAPClient Exception", e);
        }
  }
}