package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: SOAPManager
 * 
 */

import javax.xml.soap.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.util.*;
import javax.activation.*;

public class SOAPManager {
    
    private static final int MAX_BUFFER = 10000;

    /**
     * Prints Exception message and stack trace
     * prepended with passed in info String
     */
    public void printException(String info, Exception ex){
        System.out.println("\n" + info 
                            + "\nException: " + ex.getMessage()
                            + "\nStack Trace: ");
        ex.printStackTrace();
        System.out.println("\n");
    }

    /**
     * Prints Exception message and stack trace
     */
    public void printException(Exception ex){
        printException("SOAPManager Exception...", ex);
    }

    /**
     * Creates an empty SOAPMessage object
     */
    public SOAPMessage createEmptyMessage(){
        MessageFactory factory = null;
        SOAPMessage message = null;
        
        // Create a message factory           
        try{
            factory = MessageFactory.newInstance();          
        }
        catch(SOAPException e1){
            printException(e1);
        }
        
        // Create an empty SOAP Message for our request
        try{
            message = factory.createMessage();        
        }
        catch(SOAPException e1){
            printException(e1);
        }        
        
        return message;
    }

    /**
     * Dump the guts of the SOAP Message to screen.
     */
    public void printMessage(SOAPMessage msg) 
                throws SOAPException, IOException{
        FileOutputStream fos = new FileOutputStream(FileDescriptor.out);
        DataOutputStream dos = new DataOutputStream(fos);                        
        msg.writeTo(dos);
        fos.flush();
    }

    /**
     * Dump the attachment of the SOAP Message to screen
     */
    public void readAttachments(SOAPMessage msg) 
                throws SOAPException, IOException{        
        // Get an Iterator of attachments from the SOAP message
        Iterator attachments = msg.getAttachments();
        while(attachments.hasNext()){
            // Get an AttachmentPart from the Iterator
            AttachmentPart part = (AttachmentPart) attachments.next();
            
            // Print AttachmentPart ID and headers
            System.out.println("\nReading Attachment: "
                                + part.getContentId());
            Iterator headers = part.getAllMimeHeaders();
            while(headers.hasNext()){
                MimeHeader header = (MimeHeader) headers.next();
                System.out.println("\n  Header Name: " 
                                    + header.getName());
                System.out.println("\n  Header Value: " 
                                    + header.getValue());                
            }
            
            System.out.println("\n Opening Display of Image ... ");
            ImageReader ir = null;
            
            // Get attachment content and pop up its display on client
            Object content = part.getContent();
            if(content instanceof InputStream){
                ir = new ImageReader((InputStream) content);
            }
            else{
                ir = new ImageReader((java.awt.Image) content);
            }
            ir.render();
        }  
    }

    /**
     * Generates an HTTP response header to bake into response.
     * MimeHeaders are read in order to generate such a header.
     * Headers read from SOAPMessage are written to OutputStream.
     */
    public void writeHttpHeader(SOAPMessage msg, 
                                    OutputStream outStream){
        byte headerBytes[] = null;
        boolean isNotMultipart = true;
        String response = "HTTP/1.1 200 OK\n";
                
        try{
            // Read MimeHeaders        
            MimeHeaders mhs = msg.getMimeHeaders();
            Iterator it = mhs.getAllHeaders();

            // Read each MimeHeader name and value
            // and build up response String.
            while(it.hasNext()){
                MimeHeader mh = (MimeHeader) it.next();
                String name = mh.getName();
                String value = mh.getValue();
                response += name + ": " + value +"\n";
                if(value.startsWith("multipart")){
                    isNotMultipart = false;
                }
            }            
            response += "\n";
            
            System.out.println("\n\nHTTP Header\n"+response+"\n");
            
            // Only print message if has no attachments.
            if(isNotMultipart){
                printMessage(msg);
            }

            // Write the HTTP header to the output stream
            headerBytes = response.getBytes("utf-8");     
            outStream.write(headerBytes);            
        }
        catch(Exception e){
            printException(e);
        }
    }

    /**
     * Writes the SOAPMessage to an OutputStream.
     * Induces the generation and writing of
     * an HTTP header to the output stream.
     */
    public void writeMessage(SOAPMessage response, 
                              OutputStream outStream){
        try{
            if(response.saveRequired()){
                response.saveChanges();
            }
            
            writeHttpHeader(response, outStream);            
            response.writeTo(outStream);
            outStream.flush();
        }
        catch(Exception e){
            printException("Trouble writing message", e);
        }        
    }

    /**
     * Strip off HTTP header and return SOAP String
     */
    public String readMessage(InputStream msg){
        String soapString = "";
        String inputString = "";
        byte buffer[] = new byte[MAX_BUFFER];

        try{
            // While haven't read whole message
            while(inputString.indexOf("/soap-env:Envelope>") == -1){
                // read bytes from stream
                int bytesRead = msg.read(buffer, 0, MAX_BUFFER);
                // Set onto inputString
                inputString += new String(buffer, 0, bytesRead);                    
            }
            soapString = inputString.substring(
                            inputString.indexOf("<soap-env:Envelope"));
            System.out.println("SOAP Request Message...\n" + soapString);
        }
        catch(Exception e){
            printException(e);
        }
        return soapString;
    }

    /**
     * Simply returns SOAP Body from SOAPMessage
     */
    public SOAPBody getBody(SOAPMessage msg){
        SOAPBody soapBody = null;

        try{
            SOAPPart part = msg.getSOAPPart();
            SOAPEnvelope env = part.getEnvelope();
            soapBody = env.getBody();        
        }
        catch(Exception e){
            printException(e);
        }
        
        return soapBody;
    }
           
           
    /**
     * Create a SOAPMessage (Attachments, Header, Body) from a HashMap
     * and to be sent to the actor specified by the host and port     
     */
    public SOAPMessage createMessage(HashMap hashMessage, 
                                        String host, String port){ 
        // Create empty SOAP Message, get Body and create Header
        SOAPMessage msg = createEmptyMessage();
        SOAPHeader header = createHeader(msg, host, port);
        SOAPBody body = getBody(msg);        
        
        try{               
            // Get names of return objects
            Iterator names = hashMessage.keySet().iterator();
            
            // For each element in the HashMap, do this...
            while(names.hasNext()){
                // Get the name and value of the hashmap element
                String objName = (String) names.next();
                Object returnObject = hashMessage.get(objName);
                
                // If it is simply a hashmap of data, insert into body
                if(returnObject instanceof HashMap){
                    insertElement((HashMap)returnObject, body);                
                } // Otherwise, if it is a collection...
                else if(returnObject instanceof Collection){

                    // Get an iterator over the collection of HashMaps
                    Iterator elements 
                        = ((Collection) returnObject).iterator();

                    // For each HashMap of results in the Collection...
                    for(int i = 0; elements.hasNext(); ++i){

                        // Get the HashMap element to work on
                        HashMap subElement = (HashMap) elements.next();
                        HashMap xmlData = subElement;

                        // Determine if has attachments
                        String hasAttachments 
                            = (String) subElement.get("hasAttachments");

                        // If true, insert attachments and get xml data
                        if(hasAttachments.equals("true")){
                            HashMap attachments 
                               =(HashMap) subElement.get("attachments");
                            insertAttachments(attachments, msg);
                            xmlData = (HashMap) subElement.get("data");
                        }

                        // Regardless, insert XML data into body
                        insertBody(objName+":"+i, body, xmlData);
                    }
                    
                }                
            }
            
            // Save changes to message before proceed
            msg.saveChanges();
        }
        catch(Exception e){
            printException(e);
        }
        
        return msg;
    }
    
    /**
     * Insert data from the HashMap into the SOAPElement
     */
    public void insertElement(HashMap from, SOAPElement to){
        try{
            // Get iterator over names of elements in the HashMap
            Iterator names = from.keySet().iterator();
            while(names.hasNext()){
                // Get the actual SOAP element name from HashMap entry
                String objName = (String) names.next();
                // get the actual SOAP element text value from entry
                String returnObject = (String) from.get(objName);
                if(returnObject != null){
                    // Use the SOAPFactory to create a new Name instance
                    Name element 
                        = SOAPFactory.newInstance().createName(objName);
                    // Create a new child element with the Name above
                    SOAPElement child = to.addChildElement(element);
                    // Add text information for the value of the element
                    child.addTextNode(returnObject);
                }
            }
        }
        catch(Exception e){
            printException(e);
        }
        
    }

    /**
     * Insert attachment from the HashMap into the SOAPElement
     */
    public void insertAttachments(HashMap from, SOAPMessage msg){
        try{
            // Get names for the attachments from our example HashMap
            Iterator names = from.keySet().iterator();
            
            while(names.hasNext()){
                // Get the name of the current attachment
                String objName = (String) names.next();

                // Get the name of the attachment name from the HashMap
                // For this example, the name is simply a filename
                // where the attachment is stored on the server.
                String attachmentName = (String) from.get(objName);

                if(attachmentName != null){
                    // Create a data handler for the attachment and
                    //   create the attachment part with the handler
                    FileDataSource fds = 
                        new FileDataSource(attachmentName);
                    DataHandler dh = new DataHandler(fds);
                    AttachmentPart part = msg.createAttachmentPart(dh);
                    
                    // Set a MIME header type for this attachment part
                    part.setMimeHeader("Content-Type", "image/jpeg");
                    
                    // Set the Content-ID as the name read from HashMap
                    part.setContentId(objName);
                    
                    // Finally, add the attachment to the SOAP message
                    msg.addAttachmentPart(part);
                }
            }
        }
        catch(Exception e){
            printException(e);
        }        
    }
    
    /**
     * Insert data inside HashMap into SOAP Body
     */
    public void insertBody(String name, SOAPBody body, HashMap xmlData){
        try{
            SOAPFactory soapFactory = SOAPFactory.newInstance();
            Name element = soapFactory.createName(name);
            SOAPElement bodyChild = body.addBodyElement(element);                    
            insertElement(xmlData, bodyChild);
        }
        catch(Exception e){
            printException(e);
        }
    }

    /**
     * Simply returns SOAP Body from SOAPMessage
     */
    public SOAPHeader getHeader(SOAPMessage msg){
        SOAPHeader soapHeader = null;

        try{
            SOAPPart part = msg.getSOAPPart();
            SOAPEnvelope env = part.getEnvelope();
            soapHeader = env.getHeader();        
        }
        catch(Exception e){
            printException(e);
        }
        
        return soapHeader;
    }    

    
    /**
     * Create a SOAPHeader with mustUnderstand set to true
     */
    public SOAPHeader createHeader(SOAPMessage msg, 
                                    String host, String port){
        SOAPHeader headers = getHeader(msg);
        try{
            SOAPFactory soapFactory = SOAPFactory.newInstance();
            Name element = soapFactory.createName("ejava.webservices");
            
            SOAPHeaderElement header 
                = headers.addHeaderElement(element);
           header.setMustUnderstand(true);
           
           header.setActor("http://" + host + ":" + port);
        }
        catch(Exception e){
            printException(e);
        }
        return headers;
    }
    
    
    /**
     * This method parses a SOAPString into a HashMap representation
     */
    public HashMap parseMessage(String soapString){
        HashMap hashMessage = new HashMap();
        try{
            // First create a byte array stream out of string
            byte byteStream[] = soapString.getBytes();
            ByteArrayInputStream bais 
                = new ByteArrayInputStream(byteStream);
            
            // Parse input stream into a DOM document
            DocumentBuilderFactory docFactory 
                = DocumentBuilderFactory.newInstance();
            Document soapDoc 
                = docFactory.newDocumentBuilder().parse(bais);

            // Get to the header and request data
            org.w3c.dom.Node envelope = soapDoc.getFirstChild();
            org.w3c.dom.Node header = envelope.getFirstChild();
            org.w3c.dom.Node body = envelope.getFirstChild().getNextSibling();

            org.w3c.dom.Node headerElement = header.getFirstChild();
            org.w3c.dom.Node request = body.getFirstChild();
            
            String mustUnderstandString 
                = headerElement.getAttributes()
                    .getNamedItem("soap-env:mustUnderstand")
                    .getNodeValue();

            Boolean mustUnderstand = new Boolean("false");

            if(mustUnderstandString.equals("1")){
                mustUnderstand = Boolean.valueOf("true");
            }
                    
            hashMessage.put("mustUnderstand", mustUnderstand);
                        
            // Get parameters from the SOAP body request
            // and put inside the HashMap
            NodeList nl = request.getChildNodes();
            for(int i = 0; i < nl.getLength(); ++i){
                String name = nl.item(i).getNodeName();
                String value = nl.item(i).getFirstChild().getNodeValue();
                hashMessage.put(name, value);
            }
            
            // Also put the class and method type inside hashmap
            String requestType = request.getNodeName();
            int index = requestType.indexOf(":");
            String classType = requestType.substring(0,index);
            hashMessage.put("beeshirts:CLASS_NAME", classType);
            String methodType = requestType.substring(index+1);
            hashMessage.put("beeshirts:METHOD_NAME", methodType);
        }
        catch(Exception e){
            printException(e);
        }
        
        return hashMessage;
    }

    
    /**
     * Uses the Exception, failure name, actor host, and actor
     * port information to create and populate a SOAPFault
     * object which is associated with the SOAPMessage
     */
    public void createBodyFault(SOAPMessage msg, Exception exc, 
                                    String failureName,
                                    String host, String port){
        try{
            // Get the SOAP body
            SOAPFactory factory = SOAPFactory.newInstance();
            SOAPBody body = getBody(msg);
            
            // Create and populate a SOAPFault
            SOAPFault fault = body.addFault();           
            fault.setFaultString(exc.getMessage());
            fault.setFaultActor("http://" + host + ":" + port);
            fault.setFaultCode("MustUnderstand");

            // Create SOAP fault detail and attach to SOAPFault
            Detail detail = fault.addDetail();
            Name detailName = factory.createName(failureName, 
                                        "ejava.webservices.SOAPManager", 
                                        "http://beeshirts.com/soap");
            detail.addDetailEntry(detailName);
        }
        catch(Exception e){
            printException(e);
        }
    }
    
    public MimeHeaders createStandardHeaders(int size){
        MimeHeaders headers = new MimeHeaders();
        headers.addHeader("Content-Type", "text/xml");
        headers.addHeader("Content-Length", String.valueOf(size));
        return headers;
    }
}
