package ejava.webservices.rpc;

/**
 * @author Paul Perrone
 *
 * J2EE Developer's Handbook
 *
 * Package: ejava.webservices.rpc
 *
 * Type: QueryClient
 *
 * Implements a Web services client.
 *
 */

import javax.xml.rpc.*;
import javax.xml.namespace.QName;
import java.net.URL;
import java.util.prefs.Preferences;
import java.util.Vector;
import java.util.Iterator;

public class QueryClient {

  public static final String ENCODING_NAMESPACE
    = "javax.xml.rpc.encodingstyle.namespace.uri";
  public static final String ENCODING_URI
    = "http://schemas.xmlsoap.org/soap/encoding/";
  public static final String SCHEMA_NAMESPACE
    = "http://www.w3.org/2001/XMLSchema";

  private Preferences prefs = null;

  public QueryClient(){
      prefs = Preferences.userNodeForPackage(
        ejava.webservices.rpc.QueryClient.class);
  }

  public static void handleException(Exception e){
    System.out.println("<QueryClient Exception>:\n" + e.getMessage() + "\n");
    e.printStackTrace();
    System.out.println("\n\n");
  }

  public Service getOrderServiceDII(){
    Service service = null;
    try{
      QName qname = new QName(prefs.get("SERVICE_LOCALNAME", ""));

      ServiceFactory sf = ServiceFactory.newInstance();
      service = sf.createService(qname);
    }
    catch(Exception e){
      handleException(e);
    }
    return service;
  }

  public QueryService getOrderServiceStatic() {
	// Return the static QueryService in runtime-specific fashion.
	return (QueryService)(new QueryService_Impl());
  }

  public Service getOrderServiceProxy(){
    Service service = null;
    try{
      QName qname = new QName(prefs.get("SERVICE_NAMESPACE", ""),
                  prefs.get("SERVICE_LOCALNAME", ""));
      URL url = new URL(prefs.get("SERVICE_URL", ""));

      ServiceFactory sf = ServiceFactory.newInstance();
      service = sf.createService(url, qname);
    }
    catch(Exception e){
	   	System.out.println("Problem getting dynamic proxy service...");
	    handleException(e);
    }
    return service;
  }

  public void addHandlerChain(Service service){
  	// Get handler chain
	QName port = new QName(prefs.get("PORT_LOCALNAME_DII", ""));
	javax.xml.rpc.handler.HandlerRegistry
		handlerRegistry = service.getHandlerRegistry();
    java.util.List handlerChain =
    	handlerRegistry.getHandlerChain(port);

    // Create handler header info
    java.util.HashMap map = new java.util.HashMap();
    map.put("headerNamespace","http://ws.beeshirts.com/wsdl/QueryService");
    map.put("headerName","Security");
    map.put("headerPrefix","beeshirts");
    javax.xml.rpc.handler.HandlerInfo handlerInfo =
    	new  javax.xml.rpc.handler.HandlerInfo(LoginHandler.class,map,null);
    	
    // Add handler header info to the handler chain
    handlerChain.add(handlerInfo);

  }

  public Call getOrderPortDII(Service service){
    Call call = null;
    try{

      QName port = new QName(prefs.get("PORT_LOCALNAME_DII", ""));
      call = service.createCall(port);
      call.setTargetEndpointAddress(prefs.get("SERVICE_URL_DII", ""));

      call.setProperty(ENCODING_NAMESPACE, ENCODING_URI);
      call.setProperty(Call.SOAPACTION_USE_PROPERTY,new Boolean(true));
      call.setProperty(Call.SOAPACTION_URI_PROPERTY, "");

      QName stringParamType
                = new QName(SCHEMA_NAMESPACE,
                            prefs.get("OPERATION_OUTPARAM_ORDER", ""));
      call.setReturnType(stringParamType);


      QName operation = new QName(prefs.get("OPERATION_NAMESPACE", ""),
                                   prefs.get("OPERATION_LOCALNAME", ""));
      call.setOperationName(operation);
    }
    catch(Exception e){
      handleException(e);
    }
    return call;
  }

  public QueryServer getOrderPortStatic(QueryService queryService) {
	// Return the QueryServer static stub
	return queryService.getQueryServerPort();
  }

  public QueryServer getOrderPortProxy(Service service){
    QueryServer port = null;
    try{
      QName qname = new QName(prefs.get("PORT_NAMESPACE", ""),
                  prefs.get("PORT_LOCALNAME", ""));
      port = (QueryServer) service.getPort(qname, QueryServer.class);
    }
    catch(Exception e){
	   	System.out.println("Problem getting dynamic proxy port...");
    	handleException(e);
    }
    return port;
  }
  


  public String getPingResponse(Call pingServiceCall){
    String pingResponse = null;
    try{
      Object[] inParam = { };
      pingResponse = (String) pingServiceCall.invoke(inParam);
    }
    catch(Exception e){
      handleException(e);
    }
    return pingResponse;
  }

  public Order getOrder(QueryServer queryServer){
    Order order = null;
    int orderID =prefs.getInt("OPERATION_PARAMVALUE_ORDER", 1005);
    try{
      order = queryServer.findOrder(orderID);
    }
    catch(Exception e){
	   	System.out.println("Problem in getOrder(QueryServer)...");
	    handleException(e);
    }
    return order;
  }

  public Iterator getCustomers(QueryServer port){
    Vector customers = new Vector();
    float orderValue = Float.valueOf(
      prefs.get("OPERATION_PARAMVALUE_CUSTOMER", "0")).floatValue();
    try{
      port.lookupCustomers(orderValue);
      for(int i=0; i < port.length(); ++i){
		  customers.add(port.getNextCustomer());
      }
      port.resetCustomers();
    }
    catch(Exception e){
      handleException(e);
    }
    return customers.iterator();
  }

  public void createStatefulSession(QueryServer server){
	  Stub stub = (Stub) server;
      stub._setProperty(Stub.SESSION_MAINTAIN_PROPERTY,
                     	new Boolean(true)); 	
  }


  public static void main(String[] args) {
    try{
      // Construct a new query client and read preferences
      QueryClient client = new QueryClient();
	    Preferences prefs = Preferences.userNodeForPackage(
					        ejava.webservices.rpc.QueryClient.class);


      // Demonstrate invoking a DII client interface for pinging
      //    the remote service
      Service queryServiceDII = client.getOrderServiceDII();
      client.addHandlerChain(queryServiceDII);
      Call pingServiceCall = client.getOrderPortDII(queryServiceDII);
      String pingResponse = client.getPingResponse(pingServiceCall);
      System.out.println("<QueryServer.ping() - DII>:\n" + pingResponse + "\n");

      // Demonstrate invoking a static client interface for finding
      //    order information
      QueryService queryServiceStatic = client.getOrderServiceStatic();
      QueryServer queryServerStatic = client.getOrderPortStatic(queryServiceStatic);
      Order order1 = client.getOrder(queryServerStatic);
      System.out.println("<QueryServer.findOrder() - STATIC>\n");
      System.out.println("  <Order>: { "
                            + order1.getOrderID() + ", "
                            + order1.getOrderValue() + " }\n");

      // Demonstrate invoking a proxy client interface for finding
      //      order information (if serialization w/proxies supported)
      if(prefs.getBoolean("PROXY_SUPPORT", false)){
	      Service orderService = client.getOrderServiceProxy();
	      QueryServer orderPort = client.getOrderPortProxy(orderService);
	      Order order2 = client.getOrder(orderPort);
	      System.out.println("<QueryServer.findOrder() - PROXY>\n");
	      System.out.println("  <Order>: { "
    	                        + order2.getOrderID() + ", "
        	                    + order2.getOrderValue() + " }\n");
      }

      // Demonstrate session-based Web servicing over HTTP (if supported)
      if(prefs.getBoolean("SESSION_SUPPORT", false)){
      	  client.createStatefulSession(queryServerStatic);

	      Iterator customers = client.getCustomers(queryServerStatic);
    	  System.out.println("<QueryServer.lookupCustomers() - SESSION>\n");
	      while(customers.hasNext()){
    	    Customer customer = (Customer) customers.next();
        	System.out.println("  <Customer>: { "
                  + customer.getFirstName() + ", "
                  + customer.getLastName() + ", "
                  + customer.getAddress1() + ", "
                  + customer.getAddress2() + ", "
                  + customer.getCity() + ", "
                  + customer.getState() + ", "
                  + customer.getZip() + ", "
                  + customer.getPhoneNumber() + ", "
                  + customer.getEmailAddress() + " }\n");
	      }
      }
    }
    catch(Exception e){
      QueryClient.handleException(e);
    }
  }
}
