package ejava.webservices.rpc;

/**
 * @author Paul Perrone
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices.rpc
 * 
 * Type: QueryServerImpl
 * 
 * Defines a Web services endpoint implementation.
 * 
 */

import javax.xml.rpc.server.*;
import javax.servlet.http.HttpSession;
import java.sql.*;
import java.util.prefs.*;
import java.util.*;

public class QueryServerImpl implements QueryServer, ServiceLifecycle
{
   private ServletEndpointContext ctx = null;
   private static final String CUSTOMERS = "CUSTOMERS";
   private static final String INDEX = "INDEX";
   private static final String LENGTH = "LENGTH";
   private Connection connection = null;
   
   public static String CUSTOMER_QUERY 
     = "SELECT C.* FROM TSHIRTS.CUSTOMER  C, TSHIRTS.ORDERS  O, "
     + "TSHIRTS.ITEM  I WHERE C.CUSTOMER_ID = O.CUSTOMER_ID_FK AND"
     + " O.ORDER_ID = I.ORDER_ID_FK  AND I.TOTAL_PRICE > ?";
   public static String ORDER_QUERY
     = "SELECT ORDERS.order_id, ORDERS.order_date, ITEM.total_price "
     + "FROM TSHIRTS.ORDERS, TSHIRTS.ITEM, TSHIRTS.CUSTOMER WHERE "
     + "ORDERS.order_id = ITEM.order_id_fk  AND ORDERS.CUSTOMER_ID_FK "
     + "= CUSTOMER.CUSTOMER_ID AND ORDERS.order_id = ?";

   public QueryServerImpl(){
   }

   public void init(Object context){
     try {
       // Get the service context (runtime specific)
       setContext(context);

       // Get connection properties from preferences
       Preferences preferences =
               Preferences.userNodeForPackage(
                  ejava.webservices.rpc.QueryServer.class);
       String driverClass = preferences.get("DRIVER_CLASS", "");
       String dbURL   = preferences.get("DATABASE_URL","");
       String userName = preferences.get("USER_NAME","");
       String password = preferences.get("PASSWORD","");
 
       // Get JDBC connection based on preferences information
       Class.forName(driverClass);
       if(userName == null || password == null ||
          userName.length() == 0|| password.length() == 0) {
         connection = DriverManager.getConnection(dbURL);
       } else {
         connection = 
            DriverManager.getConnection(dbURL, userName,password);
       }
     } catch (ClassNotFoundException cnfEx) {
       System.err.println("Driver Class Not found " + cnfEx);
     } catch (SQLException sqlEx) {
       System.err.println("SQL Exception :" + sqlEx);
     } catch (Exception ex) {
       System.err.println("Fatal Exception :" + ex);
       ex.printStackTrace();
     }
   }

   protected void setContext(Object context){
      // Get the service context (runtime specific)
      ctx = (ServletEndpointContext) context;   
   }

   protected ServletEndpointContext getContext(){
      return ctx;
   }

   protected void setConnection(Connection conn){
     connection = conn;   
   }

   protected Connection getConnection(){
     return connection;   
   }

   public void destroy(){
      try{
      	 // Close the JDBC connection
         connection.close();
      }
      catch(SQLException sqlEx){
         sqlEx.printStackTrace();
      }
   }
   
   protected ResultSet getResultSet(String sqlQuery, Object whereValue)
        throws SQLException{
      // Get a result set based on a prepared statement
      PreparedStatement preparedStatement =
            connection.prepareStatement(sqlQuery);

      preparedStatement.setObject(1, whereValue);

      ResultSet rs = preparedStatement.executeQuery();

      return rs;
   }

   public String ping(){
   	  // Return the hello world PING info
      return "QueryService is alive! \n";
   }

   public Order findOrder(int orderID){
      Order order = new Order();
            
      // Issue order query and return order to client
      try {
        ResultSet rs = getResultSet(ORDER_QUERY, 
                                    new Integer(orderID));

        rs.next();           
        
        orderID = rs.getInt("ORDER_ID");
        double orderValue = rs.getDouble("TOTAL_PRICE");
        order.setOrderID(orderID);
        order.setOrderValue(orderValue);
      }
      catch (Exception ex){
        handleException(ex);
      }
      return order;
   }

   public void lookupCustomers(float value){
      Vector customers = new Vector();

      // Issue query and create/bind Customer for each result
      //  ...also return Vector of customer IDs
      try {
         ResultSet rs = this.getResultSet(CUSTOMER_QUERY,
                                       new Float(value));

         int length = 0;
         while (rs.next()) {
           String customerID = rs.getString("CUSTOMER_ID");
           String firstName = rs.getString("FIRST_NAME");
           String lastName = rs.getString("LAST_NAME");
           String address1 = rs.getString("ADDRESS_1");
           String address2 = rs.getString("ADDRESS_2");
           String city = rs.getString("CITY");
           String state = rs.getString("STATE");
           String phoneNumber = rs.getString("PHONE");
           String eMail = rs.getString("EMAIL");
           String zip = rs.getString("ZIP_CODE");
           Customer custImpl = new Customer();

           custImpl.setFirstName(firstName);
           custImpl.setLastName(lastName);
           custImpl.setAddress1(address1);
           custImpl.setAddress2(address2);
           custImpl.setCity(city);
           custImpl.setState(state);
           custImpl.setZip(zip);
           custImpl.setEmailAddress(eMail);
           custImpl.setPhoneNumber(phoneNumber);

           customers.addElement(custImpl);
           length++;
        }

        ctx.getHttpSession().setAttribute(CUSTOMERS, customers);
        ctx.getHttpSession().setAttribute(INDEX, new Integer(0));
        ctx.getHttpSession().setAttribute(LENGTH, new Integer(length));
       } catch (Exception ex) {
          handleException(ex);
       }
   }

  public Customer getNextCustomer(){
    Object customers = ctx.getHttpSession().getAttribute(CUSTOMERS);
    if(customers == null){
      return new Customer();
    }

    int index = 
       ((Integer) ctx.getHttpSession().getAttribute(INDEX)).intValue();
    ctx.getHttpSession().setAttribute(INDEX, new Integer(index + 1));

    return ((Customer) ((Vector) customers).get(index));
  }

  public void clearCustomers(){
    ctx.getHttpSession().setAttribute(CUSTOMERS, null);
    ctx.getHttpSession().setAttribute(INDEX, new Integer(0));
    ctx.getHttpSession().setAttribute(LENGTH, new Integer(0));
  }

  public void resetCustomers(){
     ctx.getHttpSession().setAttribute(INDEX, new Integer(0));
  }

  public int length(){
    Integer lengthInteger
       = (Integer) ctx.getHttpSession().getAttribute(LENGTH);
    return lengthInteger.intValue();
  }
  
  protected void handleException(Exception e){
    System.out.println("\n QUERY SERVER EXCEPTION:\n" +e.getMessage());
  }
}


